

#local
#code VCC_LONG
#include "vars32.s"


; -----------------------------------------
; 	32 bit arithmetics
; -----------------------------------------


; ________________________________
; complement
;
; CPL32 ( n32 -- n32 )

 macro CPL32
 	pop	hl
 	call	cpl32
	push	hl
 endm

; cpl32 ist in vcore16.s

; ________________________________
; boolean negation: !n
;
; NOT32 ( n32 -- n )

 macro NOT32
	pop	hl
 	call	not32
 endm	

not32::	ld	a,d
	or	e
	or	h
	or	l
	inc	de	; assume hlde=0 -> de=1
	ret	z
	ld	de,0
	ret

; ________________________________
; calculate sign of signed long int
;
; SIGN32 ( n32 -- n )

 macro SIGN32
	pop	hl
	call	sign32
 endm
 	
sign32::
	ld	a,h
	add	a
	jp	c,return_minus1
	or	l
	or	d
	or	e
	ret	z	; 0

	ld	de,1
	ret

; ________________________________
; convert long int to boolean
;
; BOOL32 ( n32 -- n )

 macro BOOL32 	
	pop	hl
	call	bool32
 endm

bool32:: 	
	ld	a,d
	or	e
	or	h
	or	l
	ret	z
	ld	de,1
	ret

; ________________________________
; calc msbit
;
; MSBIT32 ( n32 -- n )

 macro MSBIT32
	pop	hl		; hl = NN.hi
	call	msbit32
 endm
 
msbit32::
	ld	a,h
	or	l
	jp	z,msbit		; NN.hi = 0 -> calc msbit(NN.lo)

    ; calc msbit(NN.hi) and add 16:

	ex	hl,de		; de = nn.hi
	call	msbit
	set	4,e
	ret

; ________________________________
; extend signed i16 to i32
;
; EXTENDs32 ( i16 -- i32 )

 macro EXTENDs32
	ld	a,d
	add	a
	sbc	hl,hl
	push	hl
 endm	
	
; ________________________________
; extend unsigned u16 to u32
;
; EXTENDu32 ( u16 -- u32 )

 macro EXTENDu32
	ld	hl,0
	push	hl
 endm

; ________________________________
; calculate absolute value of signed long value
; return value is uint32, because abs($80000000) = $80000000
;
; ABS32 ( n32 -- n32 )

 macro ABS32
	pop	hl
	bit	7,h
	call	nz,neg32
	push	hl
 endm

; ________________________________
; negate
;
; NEG32 ( n32 -- n32 )

 macro NEG32
	pop	hl
 	call	neg32
	push	hl
 endm

neg32::	xor	a
	sub	e
	ld	e,a
	ld	a,0
	sbc	d
	ld	d,a
	ld	a,0
	sbc	l
	ld	l,a
	ld	a,0
	sbc	h
	ld	h,a	
	ret

; ________________________________
; swap bytes - swap endianess
;
; SWAPBYTES32 ( n32 -- n32 )

 macro SWAPBYTES32
 	pop	hl
	call	swapbytes32
	push	hl
 endm

swapbytes32::
	ld	a,h
	ld	h,e
	ld	e,a
	ld	a,l
	ld	l,d
	ld	d,a
	ret 	 

; ________________________________
; add 32 bit values
;
; ADD32 ( n1.l n2.l -- n3.l )

 macro ADD32
 	call	add32
 endm
 	
add32::	pop	iy		; iy=ret addr
	pop	hl		; hl = n2.hi, de = n2.lo, stack=n1.lo,n1.hi
	ex	hl,(sp)		; hl = n1.lo, de = n2.lo, stack=n2.hi,n1.hi
	add	hl,de		; hl = n3.lo,             stack=n2.hi,n1.hi
	pop	de		; hl = n3.lo, de = n2.hi, stack=n1.hi
	ex	hl,(sp)		; hl = n1.hi, de = n2.hi, stack=n3.lo
	adc	hl,de		; hl = n3.hi,             stack=n3.lo
	pop	de		; hl = n3.hi, de = n3.lo
	push	hl		; de = n3.lo,             stack=n3.hi
	jp	iy

; ________________________________
; subtract 32 bit values n1 - n2
;
; SUB32 ( n1.l n2.l -- n3.l )

 macro SUB32
	call	sub32
 endm
 
sub32::	pop	iy		; ret addr
	pop	hl		; hl = n2.hi, de = n2.lo, stack=n1.lo,n1.hi
	ex	hl,(sp)		; hl = n1.lo, de = n2.lo, stack=n2.hi,n1.hi
	and	a
	sbc	hl,de		; hl = n3.lo,             stack=n2.hi,n1.hi
	pop	de		; hl = n3.lo, de = n2.hi, stack=n1.hi
	ex	hl,(sp)		; hl = n1.hi, de = n2.hi, stack=n3.lo
	sbc	hl,de		; hl = n3.hi,             stack=n3.lo
	pop	de		; hl = n3.hi, de = n3.lo
	push	hl		; de = n3.lo,             stack=n3.hi
	jp	iy
	


; ________________________________
; add offset to long pointer
;
; ADDp32i ( l* n -- l* )

 macro ADDp32i
	pop	hl	
	ex	hl,de
	add	hl,hl		; hl = offset*2
	add	hl,hl		; hl = offset*4
	add	hl,de
	ex	de,hl
 endm

; ________________________________
; subtract offset from long pointer
;
; SUBp32i ( l* n -- l* )	

 macro SUBp32i
	pop	hl
	call	subp32i
 endm	

subp32i::
	ex	hl,de
	add	hl,hl		
	ex	hl,de		; de = offset*2
	ex	hl,de		; de = offset*4
	and	a
	sbc	hl,de
	ex	de,hl
	ret

; ________________________________
; subtract two long pointers
; the result is signed
;
; SUBp32i ( l1* l2* -- i16 )	

 macro SUBp32p
	pop	hl
	call	subp32p
 endm
	
subp1326p::
	and	a
	sbc	hl,de		; hl = p1-p2
	ex	hl,de		; de = p1-p2
	;jr	sr2s
	
; ________________________________
; signed shift right by two bits
;
; SR2s ( n -- n )

 macro SR2s
 	call	sr2s
 endm
 
sr2s:: 	sra	d
	rr	e
	sra	d
	rr	e
	ret

; ________________________________
; reset bits in long word
;
; AND32 ( n32 n32 -- n32 )

 macro AND32
 	call	and32
 endm
 	
and32::	pop	iy		; ret addr
	pop	hl		; de=n2lo, hl=n2hi, stack:n1hi n1lo
	ex	hl,(sp)		; de=n2lo, hl=n1lo, stack:n1hi n2hi
	ld	a,h
	and 	d
	ld	h,a
	ld	a,l
	and	e
	ld	l,a		; hl=reslo, stack:n1hi n2hi
	pop	de		; de=n2hi, hl=reslo, stack:n1hi 
	ex	hl,(sp)		; de=n2hi, hl=n1hi, stack: reslo
	ld	a,h
	and 	d
	ld	h,a
	ld	a,l
	and	e
	ld	l,a		; hl=reshi, stack:reslo
	pop	de
	push	hl
	jp	iy

; ________________________________
; set bits in long word
;
; OR32 ( n32 n32 -- n32 )

 macro OR32
 	call	or32
 endm
 
or32::	pop	iy		; ret addr
	pop	hl		; de=n2lo, hl=n2hi, stack:n1hi n1lo
	ex	hl,(sp)		; de=n2lo, hl=n1lo, stack:n1hi n2hi
	ld	a,h
	or 	d
	ld	h,a
	ld	a,l
	or	e
	ld	l,a		; hl=reslo, stack:n1hi n2hi
	pop	de		; de=n2hi, hl=reslo, stack:n1hi 
	ex	hl,(sp)		; de=n2hi, hl=n1hi, stack: reslo
	ld	a,h
	or 	d
	ld	h,a
	ld	a,l
	or	e
	ld	l,a		; hl=reshi, stack:reslo
	pop	de
	push	hl
	jp	iy

; ________________________________
; toggle bits in long word
;
; XOR32 ( n32 n32 -- n32 )

 macro XOR32
 	call	xor32
 endm

xor32::	pop	iy		; ret addr
	pop	hl		; de=n2lo, hl=n2hi, stack:n1hi n1lo
	ex	hl,(sp)		; de=n2lo, hl=n1lo, stack:n1hi n2hi
	ld	a,h
	xor 	d
	ld	h,a
	ld	a,l
	xor	e
	ld	l,a		; hl=reslo, stack:n1hi n2hi
	pop	de		; de=n2hi, hl=reslo, stack:n1hi 
	ex	hl,(sp)		; de=n2hi, hl=n1hi, stack: reslo
	ld	a,h
	xor 	d
	ld	h,a
	ld	a,l
	xor	e
	ld	l,a		; hl=reshi, stack:reslo
	pop	de
	push	hl
	jp	iy

; ________________________________
; shift long value left once
;
; SL132 ( n32 -- n32 )

 macro SL132
 	pop	hl
	ex	de,hl
	add	hl,hl		; SL the low word
	ex	de,hl
	adc	hl,hl		; SL the high word
	push	hl
 endm

; ________________________________
; shift unsigned long value right once
;
; SR1u32 ( n32 -- n32 )

 macro SR1u32
 	pop	hl
	call	sr1u32
	push	hl
 endm
 
sr1u32::srl	h		; SR the high word
	rr	l
	rr	d		; SR the low word
	rr	e
	ret

; ________________________________
; shift signed long value right once
;
; SR1s32 ( n32 -- n32 )

 macro SR1s32
 	pop	hl
 	call	sr1s32
	push	hl
 endm
 
sr1s32::sra	h		; SR the high word
	rr	l
	rr	d		; SR the low word
	rr	e
	ret



; ________________________________
; shift long value left N bits 
;
; SL32 ( n32 n -- n32 )

 macro SL32
 	ld	a,e
	pop	de
	pop	hl
 	call	sl32
	push	hl
 endm

; HLDE <<= A

sl32::	bit	4,a
	jr	z,1$
	ex	hl,de
	ld	de,0		; hlde = hlde << 16
1$:	bit	3,a
	jr	z,2$
	ld	h,l
	ld	l,d
	ld	d,e
	ld	e,0		; hlde = hlde << 8
2$:	and	0x07
	ret	z

3$:	sla	e
	rl 	d
	adc	hl,hl
	dec	a
	jr 	nz,3$
	ret			; result in hlde
	
; ________________________________
; shift unsigned long value right by N bits 
;
; SRu32 ( n32 n -- n32 )

 macro SRu32
 	ld	a,e
	pop	de
	pop	hl
 	call	sru32
	push	hl
 endm

; HLDE >>= A

sru32:	bit	4,a
	jr	z,1$
	ex	de,hl
	ld	hl,0		; hlde = hlde >> 16
1$:	bit	3,a
	jr	z,2$
	ld 	e,d
	ld 	d,l
	ld 	l,h
	ld 	h,0		; hlde = hlde >> 8             
2$:	and	0x07
	ret	z
	
3$:	srl	h
	rr 	l
	rr 	d
	rr 	e
	dec	a
	jr 	nz,3$
	ret	

; ________________________________
; shift signed long value right by N bits
;
; SRs32 ( n32 n -- n32 )

 macro SRs32
 	ld	a,e
	pop	de
	pop	hl
 	call	srs32
	push	hl
 endm

; HLDE >>= A

srs32:	bit	7,h
	jr	nz,sru32

	bit	4,a
	jr	z,1$
	ex	de,hl
	ld	hl,0xFFFF	; hlde = hlde >> 16
1$:	bit	3,a
	jr	z,2$
	ld 	e,d
	ld 	d,l
	ld 	l,h
	ld 	h,0xFF		; hlde = hlde >> 8
2$:	and	0x07
	ret	z

3$:	sra	h
	rr 	l
	rr 	d
	rr 	e
	dec	a
	jr 	nz,3$
	ret

 
 
; -----------------------------------------
; 	Equality Compare
; -----------------------------------------

; ______________________________________
; compare two long values with mixed signedness
; either signed - unsigned or vice versa
; 
; EQx32 ( n32 n32 -- f )

 macro EQx32
 	call	eqx32
 endm	

eqx32::	pop	iy
	pop	hl		; hl = n2hi, de = n2lo, stack: n1hi n1lo
	ex	hl,(sp)		; hl = n1lo, de = n2lo, stack: n1hi n2hi
	and	a
	sbc	hl,de		
	pop	hl
	pop	de
	jr	nz,return_false		; low words NE
	sbc	hl,de
	jr	nz,return_false		; high words NE
	ex	hl,de
	jr	return_hl_pos		; bit15=0 => equal, bit15=1 => not equal!

; ______________________________________
; compare two long values with mixed signedness
; either signed - unsigned or vice versa
; 
; NEx32 ( n32 n32 -- f )

 macro NEx32
 	call	nex32
 endm

nex32::	pop	iy
	pop	hl		; hl = n2hi, de = n2lo, stack: n1hi n1lo
	ex	hl,(sp)		; hl = n1lo, de = n2lo, stack: n1hi n2hi
	and	a
	sbc	hl,de		
	pop	hl
	pop	de
	jr	nz,return_true		; low words NE
	sbc	hl,de
	jr	nz,return_true		; high words NE
	ex	hl,de
	jp	return_hl_neg		; bit15=0 => equal, bit15=1 => not equal!

; ______________________________________
; compare two values: n1 == n2
; either signed - signed or
; unsigned - unsigned
;
; EQ32 ( n32 n32 -- f )

 macro EQ32
 	call	eq32		
 endm

eq32::	pop	iy		; ret addr
	pop	hl		; hl = n2hi, de = n2lo, stack: n1hi n1lo
	ex	hl,(sp)		; hl = n1lo, de = n2lo, stack: n1hi n2hi
	and	a
	sbc	hl,de
	pop	hl
	pop	de
	jr	nz,return_false
	sbc	hl,de
	jr	nz,return_false

return_true:
	ld	de,1
	jp	iy

; ______________________________________
; compare two values: n1 != n2
; either signed - signed or
; unsigned - unsigned
;
; NE32 ( n32 n32 -- f )

 macro NE32
 	call	ne32
 endm
 	
ne32::	pop	iy		; ret addr
	pop	hl		; hl = n2hi, de = n2lo, stack: n1hi n1lo
	ex	hl,(sp)		; hl = n1lo, de = n2lo, stack: n1hi n2hi
	and	a
	sbc	hl,de
	pop	hl
	pop	de
	jr	nz,return_true
	sbc	hl,de
	jr	nz,return_true

return_false:
	ld	de,0
	jp	iy



; -----------------------------------------
; 	Unsigned Compare
; -----------------------------------------

; ______________________________________
; compare two unsigned values: n1 < n2
;
; LTu32 ( n32 n32 -- f )

 macro LTu32
 	call	ltu32
 endm
 	
ltu32::	pop	iy
	pop	hl		; hl=n2hi, de=n2lo, stack:n1hi n1lo
ltu32a:	ex	hl,(sp)		; hl=n1lo, de=n2lo, stack:n1hi n2hi
	xor	a		; cy=0, a=0
	sbc	hl,de		; cy=cy, stack:n1hi n2hi
	pop	de		; de=n2hi
	pop	hl		; hl=n1hi
	sbc	hl,de		; cy => n1hi - n2hi - cy < 0 => n1 < n2 => LT 
	jr	return_cy
	
; ______________________________________
; compare two unsigned values: n1 ≥ n2
;
; GEu32 ( n32 n32 -- f )

 macro GEu32
 	call	geu32
 endm
 
geu32::	pop	iy
	pop	hl		; hl=n2hi, de=n2lo, stack:n1hi n1lo
geu32a:	ex	hl,(sp)		; hl=n1lo, de=n2lo, stack:n1hi n2hi
	xor	a		; cy=0, a=0
	sbc	hl,de		; cy=cy, stack:n1hi n2hi
	pop	de		; de=n2hi
	pop	hl		; hl=n1hi
	sbc	hl,de		; nc => n1hi - n2hi - cy ≥ 0 => n1 ≥ n2 => GE
	jr	return_nc
	
; ______________________________________
; compare two unsigned values: n1 > n2
;
; GTu32 ( n32 n32 -- f )

 macro GTu32
 	call	gtu32
 endm
 
gtu32::	pop	iy
	pop	hl		; hl=n2hi, de=n2lo, stack:n1hi n1lo
gtu32a:	ex	hl,(sp)		; hl=n1lo, de=n2lo, stack:n1hi n2hi
	ex	hl,de
	xor	a		; cy=0, a=0
	sbc	hl,de		; cy=cy, stack:n1hi n2hi
	pop	hl		; hl=n2hi
	pop	de		; de=n1hi
	sbc	hl,de		; cy => n2hi - n1hi - cy < 0 => n2 < n1 => GT 
	jr	return_cy

; ______________________________________
; compare two unsigned values: n1 ≤ n2
;
; LEu32 ( n32 n32 -- f )

 macro LEu32
 	call	leu32
 endm
 
leu32::	pop	iy
	pop	hl		; hl=n2hi, de=n2lo, stack:n1hi n1lo
leu32a:	ex	hl,(sp)		; hl=n1lo, de=n2lo, stack:n1hi n2hi
	ex	hl,de
	xor	a		; cy=0, a=0
	sbc	hl,de		; cy=cy, stack:n1hi n2hi
	pop	hl		; hl=n2hi
	pop	de		; de=n1hi
	sbc	hl,de		; cy => n2hi - n1hi - cy < 0 => n2 < n1 => GT 
	jr	return_nc



; -----------------------------------------
; 	Signed Compare
; -----------------------------------------

; ______________________________________
; compare two signed values: n1 < n2
;
; LTs32 ( n32 n32 -- f )

 macro LTs32
 	call	lts32
 endm

lts32::	ld	hl,7		; de = n2lo, stack: n1hi n1lo n2hi retaddr
	add	hl,sp
	ld	a,(hl)		; a = n1hhi
	
	pop	iy		; ret addr
	pop	hl		; hl = n2hi
	xor	h
	jp	p,ltu32a		; same sign => do unsigned compare
	;jr	drop2_return_hl_pos	; different sign => n1 LT n2 is true if n2 pos

drop2_return_hl_pos:
	pop	af
	pop	af
return_hl_pos:
	xor	a		; a=0 
	add	hl,hl		; cy = hl.vz
return_nc:			; ATTN: in: a=0 !!!
	ccf			; cy => n1-n2<0 => n1<n2 => ge=no => ccf
	ld	d,a		; d=0
	adc	a,a		; nc=>0, cy=>1
	ld	e,a		; de=0|1
	jp	iy

; ______________________________________
; compare two signed values: n1 ≥ n2
;
; GEs32 ( n32 n32 -- f )

 macro GEs32
 	call	ges32
 endm

ges32::	ld	hl,7		; de = n2lo, stack: n1hi n1lo n2hi
	add	hl,sp
	ld	a,(hl)		; a = n1hhi
	
	pop	iy		; ret addr
	pop	hl		; hl = n2hi
	xor	h
	jp 	p,geu32a		; same sign => do unsigned compare
	;jp	m,drop2_return_hl_neg	; different sign => n1 ≥ n2 is true if n2 neg
	
drop2_return_hl_neg:
	pop	af
	pop	af
return_hl_neg:
	xor	a		; a=0 
	add	hl,hl		; cy = hl.vz
return_cy:			; ATTN: in: a=0 !!!
	ld	d,a		; d=0
	adc	a,a		; nc=>0, cy=>1
	ld	e,a		; de=0|1
	jp	iy

; ______________________________________
; compare two signed values: n1 > n2
;
; GTs32 ( n32 n32 -- f )

 macro GTs32
 	call	gts32
 endm

gts32::	ld	hl,7		; de = n2lo, stack: n1hi n1lo n2hi
	add	hl,sp
	ld	a,(hl)		; a = n1hhi
	
	pop	iy
	pop	hl		; hl = n2hi
	xor	h
	jp	p,gtu32a		; same sign => unsigned compare
	jr	drop2_return_hl_neg	; different sign => n1 > n2 is true if n2 neg
	
; ______________________________________
; compare two signed values: n1 ≤ n2
;
; LEs32 ( n32 n32 -- f )

 macro LEs32
 	call	les32
 endm
 
les32::	ld	hl,7		; de = n2lo, stack: n1hi n1lo n2hi
	add	hl,sp
	ld	a,(hl)		; a = n1hhi
	
	pop	iy
	pop	hl		; hl = n2hi
	xor	h
	jp	p,leu32a		; same sign => unsigned compare
	jr	drop2_return_hl_pos	; different sign => n1 ≤ n2 is true if n2 pos



; -----------------------------------------
; 	Mixed Signedness Compare
; -----------------------------------------

; ______________________________________
; compare two values with mixed signedness u32 > i32
;
; GTus32 ( u32 i32 -- f )

 macro GTus32
 	call	gtus32
 endm
 
gtus32::
	ld	hl,7		; de = n2lo, stack: n1hi n1lo n2hi
	add	hl,sp
	ld	a,(hl)		; a = n1hhi
	
	pop	iy
	pop	hl			; hl = n2hi
	or	h			; u16≥0x8000 or i16<0 ?
	jp	p,gtu32a		; no => unsigned compare
	;jr	drop2_return_true	; yes => u32 > i32 true  

drop2_return_true:
	pop	af
	pop	af
	ld	de,1
	jp	iy
	
; ______________________________________
; compare two values with mixed signedness i32 > u32
;
; GTsu32 ( i32 u32 -- f )

 macro GTsu32
 	call	gtsu32
 endm
 
gtsu32::ld	hl,7		; de = n2lo, stack: n1hi n1lo n2hi
	add	hl,sp
	ld	a,(hl)		; a = n1hhi
	
	pop	iy
	pop	hl			; hl = n2hi
	or	h			; u16≥0x8000 or i16<0 ?
	jp	p,gtu32a		; no => unsigned compare
	;jr	drop2_return_false	; yes => i32 > u32 false

drop2_return_false:
	pop	af
	pop	af
	ld	de,0
	jp	iy
	
; ______________________________________
; compare two values with mixed signedness i32 < u32
;
; LTsu32 ( i32 u32 -- f )

 macro LTsu32
 	call	ltsu32
 endm
 
ltsu32::ld	hl,7		; de = n2lo, stack: n1hi n1lo n2hi
	add	hl,sp
	ld	a,(hl)		; a = n1hhi
	
	pop	iy
	pop	hl			; hl = n2hi
	or	h			; u16≥0x8000 or i16<0 ?
	jp	p,ltu32a		; no => unsigned compare
	jr	drop2_return_true	; yes => i32 < u32 true
	
; ______________________________________
; compare two values with mixed signedness u32 < i32
;
; LT32us ( u32 i32 -- f )

 macro LTus32
 	call	ltus32
 endm
 
ltus32::ld	hl,7		; de = n2lo, stack: n1hi n1lo n2hi
	add	hl,sp
	ld	a,(hl)		; a = n1hhi
	
	pop	iy
	pop	hl			; hl = n2hi
	or	h			; u16≥0x8000 or i16<0 ?
	jp	p,ltu32a		; no => unsigned compare
	jr	drop2_return_true	; yes => u32 < i32 false
	
; ______________________________________
; compare two values with mixed signedness u32 ≥ i32
;
; GEus32 ( u32 i32 -- f )

 macro GEus32
 	call	geus32
 endm
 
geus32::ld	hl,7		; de = n2lo, stack: n1hi n1lo n2hi
	add	hl,sp
	ld	a,(hl)		; a = n1hhi
	
	pop	iy
	pop	hl			; hl = n2hi
	or	h			; u16≥0x8000 or i16<0 ?
	jp	p,geu32a		; no => unsigned compare
	jp	drop2_return_true	; yes => u32 ≥ i32 true
	
; ______________________________________
; compare two values with mixed signedness i32 ≥ u32
;
; GEsu32 ( i32 u32 -- f )

 macro GEsu32
 	call	gesu32
 endm
 
gesu32::ld	hl,7		; de = n2lo, stack: n1hi n1lo n2hi
	add	hl,sp
	ld	a,(hl)		; a = n1hhi
	
	pop	iy
	pop	hl			; hl = n2hi
	or	h			; u16≥0x8000 or i16<0 ?
	jp	p,geu32a		; no => unsigned compare
	jr	drop2_return_false	; yes => i32 ≥ u32 false

; ______________________________________
; compare two values with mixed signedness i32 ≤ u32
;
; LEsu32 ( i32 u32 -- f )

 macro LEsu32
 	call	lesu32
 endm
 
lesu32::ld	hl,7		; de = n2lo, stack: n1hi n1lo n2hi
	add	hl,sp
	ld	a,(hl)		; a = n1hhi
	
	pop	iy
	pop	hl			; hl = n2hi
	or	h			; u16≥0x8000 or i16<0 ?
	jp	p,leu32a		; no => unsigned compare
	jp	drop2_return_true	; yes => i32 ≤ u32 true
	
; ______________________________________
; compare two values with mixed signedness u32 ≤ i32
;
; LEus32 ( u32 i32 -- f )

 macro LEus32
 	call	leus32
 endm
 
leus32::ld	hl,7		; de = n2lo, stack: n1hi n1lo n2hi
	add	hl,sp
	ld	a,(hl)		; a = n1hhi
	
	pop	iy
	pop	hl			; hl = n2hi
	or	h			; u16≥0x8000 or i16<0 ?
	jp	p,leu32a		; no => unsigned compare
	jp	drop2_return_false	; yes => u32 ≤ i32 false
	


; -----------------------------------------
; 	Incr / Decr
; -----------------------------------------

; ________________________________
; increment long value
;
; INCR32 ( n32 -- n32 )

 macro INCR32
 	pop	hl
	call	incr32
	push	hl
 endm

; incr hlde:
incr32::inc	de
	ld	a,d
	or	e
	ret	nz
	inc	hl
	ret
	
; ________________________________
; decrement long value
;
; DECR32 ( n32 -- n32 )

 macro DECR32
 	pop	hl
	call	decr32
	push	hl
 endm

; decr hlde:
decr32::ld	a,d
	or	e	
	dec	de
	ret	nz
	dec	hl
	ret
	
; ________________________________
; increment long variable
;
; PPSET32 ( &n32 -- )

 macro PPSET32
 	call	ppset32
	pop	de
 endm	
	
ppset32::
	ex	de,hl		; hl->var
	
incr:	inc	(hl)
	ret	nz

	inc	hl
	inc	(hl)
	ret	nz

	inc	hl
	inc	(hl)
	ret	nz

	inc	hl
	inc	(hl)
	ret

; ________________________________
; decrement long variable
;
; MMSET32 ( &n32 -- )

 macro MMSET32
 	call	mmset32
	pop	de
 endm
 	
mmset32::
	ex	de,hl		; hl->var

decr:	ld	a,(hl)
	sub	1
	ld	(hl),a
	ret	nc		; else a = 0xFF
	
	inc	hl	
	dec	(hl)
	cp	(hl)
	ret	nz

	inc	hl
	dec	(hl)
	cp	(hl)
	ret	nz

	inc	hl
	dec	(hl)
	ret

; ________________________________
; get value with post-increment
;
; PEEKPP32 ( &n32 -- n32 )

 macro PEEKPP32
 	call	peekpp32
 endm
 
peekpp32::
	ld	hl,3
	add	hl,de		; hl -> var.hhi
	
	ld	e,(hl)
	ld	d,(--hl)
	
	pop	af
	push	de		; stack: old value.hi
	push	af
	
	ld	e,(--hl)
	ld	d,(--hl)	; d = old value.lo, hl -> var
	
	jr	incr

; ________________________________
; get value with post-decrement
;
; PEEKMM32 ( &n32 -- n32 )

 macro PEEKMM32
 	call	peekmm32	
 endm
 
peekmm32::
	ld	hl,3
	add	hl,de		; hl -> var.hhi
	
	ld	e,(hl)
	ld	d,(--hl)
	
	pop	af
	push	de		; stack: old value.hi
	push	af
	
	ld	e,(--hl)
	ld	d,(--hl)	; d = old value.lo, hl -> var
	
	jr	decr

; ________________________________
; get value with pre-decrement
;
; MMPEEK32 ( &n32 -- n32 )

 macro MMPEEK32
 	call	mmpeek32
 endm
  
mmpeek32::
	ld	hl,de		; hl->var, de->var
	call	decr		; decr(hl++)
	ex	hl,de		; hl->var
	jp	peek32

; ________________________________
; get value with pre-increment
;
; PPPEEK32 ( &n32 -- n32 )

 macro PPPEEK32
 	call	pppeek32
 endm

pppeek32::
	ld	hl,de		; hl->var, de->var
	call	incr		; incr(hl++)
	ex	hl,de		; hl->var
	jp	peek32



; -----------------------------------------
; 	Arithmetic Assignments
; -----------------------------------------

; ________________________________
; mask 32 bit variable with AND
;
; ANDGL32 ( n32 &var -- )

 macro ANDGL32
 	call	andgl32
 endm

andgl32:: 			; de -> var
	pop	iy		; iy = ret addr
	
	pop	hl		; hl = n.lo

	ld	a,(de)
	and	l
	ld	(de++),a

	ld	a,(de)
	and	h
	ld	(de++),a

	pop	hl		; hl = n.hi

	ld	a,(de)
	and	l
	ld	(de++),a

	ld	a,(de)
	and	h
	ld	(de),a

	pop	de		; new Top
	jp	iy		; return

; ________________________________
; mask 32 bit variable with OR
;
; ORGL32 ( n32 &var -- )

 macro ORGL32
 	call	orgl32
 endm
 
orgl32:: 			; de -> var
	pop	iy		; iy = ret addr
	
	pop	hl		; hl = n.lo

	ld	a,(de)
	or	l
	ld	(de++),a

	ld	a,(de)
	or	h
	ld	(de++),a

	pop	hl		; hl = n.hi

	ld	a,(de)
	or	l
	ld	(de++),a

	ld	a,(de)
	or	h
	ld	(de),a

	pop	de		; new Top
	jp	iy		; return

; ________________________________
; mask 32 bit variable with XOR
;
; XORGL32 ( n32 &var -- )

 macro XORGL32
 	call	xorgl32
 endm
 
xorgl32:: 			; de -> var
	pop	iy		; iy = ret addr
	
	pop	hl		; hl = n.lo

	ld	a,(de)
	xor	l
	ld	(de++),a

	ld	a,(de)
	xor	h
	ld	(de++),a

	pop	hl		; hl = n.hi

	ld	a,(de)
	xor	l
	ld	(de++),a

	ld	a,(de)
	xor	h
	ld	(de),a

	pop	de		; new Top
	jp	iy		; return

; ________________________________
; add 32 bit value to 32 bit variable
;
; ADDGL32 ( n32 &var -- )

 macro ADDGL32
 	call	addgl32
 endm
 
addgl32:: 			; de -> var
	pop	iy		; iy = ret addr
	
	pop	hl		; hl = n.lo

	ld	a,(de)
	add	l
	ld	(de++),a

	ld	a,(de)
	adc	h
	ld	(de++),a

	pop	hl		; hl = n.hi

	ld	a,(de)
	adc	l
	ld	(de++),a

	ld	a,(de)
	adc	h
	ld	(de),a

	pop	de		; new Top
	jp	iy		; return

; ________________________________
; subtract from long variable
;
; SUBGL32 ( n32 &var -- )

 macro SUBGL32
 	call	subgl32
 endm
 
subgl32:: 			; de -> var
	pop	iy		; iy = ret addr
	
	pop	hl		; hl = n.lo

	ld	a,(de)
	sub	l
	ld	(de++),a

	ld	a,(de)
	sbc	h
	ld	(de++),a

	pop	hl		; hl = n.hi

	ld	a,(de)
	sbc	l
	ld	(de++),a

	ld	a,(de)
	sbc	h
	ld	(de),a

	pop	de		; new Top
	jp	iy		; return

; ________________________________
; shift long variable left N bits
;
; SLGL32 ( n &var -- )

 macro SLGL32
 	call	slgl32
 endm

slgl32::			; de -> var
	pop	iy		; iy = ret addr

	ex	hl,de		; hl -> var
	pop	de		; e=shift
	ld	a,e		; a=shift
	push	hl		; stack:&var
	
	push	af		; shift
	ld	e,(hl++)
	ld	d,(hl++)
	ld	a,(hl++)
	ld	h,(hl)
	ld	l,a		; hlde = var
	pop	af		; shift

	call	sl32		; hlde = hlde << a

	ex	hl,(sp)		; hl->var, de=n.lo, stack:n.hi
	jp	poke32b

; ________________________________
; shift signed long variable right N bits
;
; SRGLs32 ( n16 &var -- )

 macro SRGLs32
 	call	srgls32
 endm

srgls32::			; de -> var
	pop	iy		; iy = ret addr

	ex	hl,de		; hl -> var
	pop	de		; e=shift
	ld	a,e		; a=shift
	push	hl		; stack:&var
	
	push	af		; shift
	ld	e,(hl++)
	ld	d,(hl++)
	ld	a,(hl++)
	ld	h,(hl)
	ld	l,a		; hlde = var
	pop	af		; shift

	call	srs32		; hlde = hlde >> a

	ex	hl,(sp)		; hl->var, de=n.lo, stack:n.hi
	jp	poke32b

; ________________________________
; shift unsigned long variable right N bits
;
; SRGLu32 ( n16 &var -- )

 macro SRGLu32
 	call	srglu32
 endm

srglu32::			; de -> var
	pop	iy		; iy = ret addr

	ex	hl,de		; hl -> var
	pop	de		; e=shift
	ld	a,e		; a=shift
	push	hl		; stack:&var
	
	push	af		; shift
	ld	e,(hl++)
	ld	d,(hl++)
	ld	a,(hl++)
	ld	h,(hl)
	ld	l,a		; hlde = var
	pop	af		; shift

	call	sru32		; hlde = hlde >> a

	ex	hl,(sp)		; hl->var, de=n.lo, stack:n.hi
	jp	poke32b








; ________________________________
;#include "mul32.s"
;#include "div32.s"
	


	
	




#endlocal



